@@ -3,15 +3,13 @@
 
 import os
 import os.path
-import warnings
 
 from pathlib import Path
-from subprocess import call
+from subprocess import check_call
 
 from click.testing import CliRunner
 from twisted.trial.unittest import TestCase
 
-from towncrier import check
 from towncrier.build import _main as towncrier_build
 from towncrier.check import _main as towncrier_check
 
@@ -28,7 +26,7 @@ def create_project(
     setup_simple_project(pyproject_path=pyproject_path, extra_config=extra_config)
     Path("foo/newsfragments/123.feature").write_text("Adds levitation")
     initial_commit(branch=main_branch)
-    call(["git", "checkout", "-b", "otherbranch"])
+    check_call(["git", "checkout", "-b", "otherbranch"])
 
 
 def commit(message):
@@ -37,8 +35,17 @@ def commit(message):
     There must be uncommitted changes otherwise git will complain:
     "nothing to commit, working tree clean"
     """
-    call(["git", "add", "."])
-    call(["git", "commit", "-m", message])
+    check_call(["git", "add", "."])
+    check_call(["git", "commit", "-m", message])
+
+
+def stage():
+    """Stage a commit to the repo in the current working directory
+
+    There must be uncommitted changes otherwise git will complain:
+    "nothing to commit, working tree clean"
+    """
+    check_call(["git", "add", "."])
 
 
 def initial_commit(branch="main"):
@@ -50,11 +57,11 @@ def initial_commit(branch="main"):
     """
     # --initial-branch is explicitly set to `main` because
     # git has deprecated the default branch name.
-    call(["git", "init", f"--initial-branch={branch}"])
+    check_call(["git", "init", f"--initial-branch={branch}"])
     # Without ``git config` user.name and user.email `git commit` fails
     # unless the settings are set globally
-    call(["git", "config", "user.name", "user"])
-    call(["git", "config", "user.email", "user@example.com"])
+    check_call(["git", "config", "user.name", "user"])
+    check_call(["git", "config", "user.email", "user@example.com"])
     commit("Initial Commit")
 
 
@@ -156,8 +163,8 @@ class TestChecker(TestCase):
             with open(file_path, "w") as f:
                 f.write("import os")
 
-            call(["git", "add", "foo/somefile.py"])
-            call(["git", "commit", "-m", "add a file"])
+            check_call(["git", "add", "foo/somefile.py"])
+            check_call(["git", "commit", "-m", "add a file"])
 
             result = runner.invoke(towncrier_check, ["--compare-with", "master"])
 
@@ -204,6 +211,41 @@ class TestChecker(TestCase):
                 (result.output, str(fragment_path)),
             )
 
+    def test_fragment_exists_and_staged(self):
+        """A fragment exists and is added in staging. Pass only if staging on the command line"""
+        runner = CliRunner()
+
+        with runner.isolated_filesystem():
+            create_project(
+                "pyproject.toml",
+                main_branch="master",
+                extra_config="[[tool.towncrier.type]]\n"
+                'directory = "feature"\n'
+                'name = "Features"\n'
+                "showcontent = true\n"
+                "[[tool.towncrier.type]]\n"
+                'directory = "sut"\n'
+                'name = "System Under Test"\n'
+                "showcontent = true\n",
+            )
+
+            file_path = "foo/somefile.py"
+            write(file_path, "import os")
+
+            commit("add some files for test initialization")
+
+            fragment_path = Path("foo/newsfragments/1234.feature").absolute()
+            write(fragment_path, "Adds gravity back")
+            stage()
+
+            result = runner.invoke(towncrier_check, ["--compare-with", "master"])
+
+            self.assertEqual(1, result.exit_code)
+            result = runner.invoke(
+                towncrier_check, ["--staged", "--compare-with", "master"]
+            )
+            self.assertEqual(0, result.exit_code)
+
     def test_fragment_exists_and_in_check(self):
         """
         A fragment that exists but is not marked as check=False is
@@ -245,7 +287,12 @@ class TestChecker(TestCase):
         """
         No failure when output is piped causing None encoding for stdout.
         """
-        runner = CliRunner()
+        try:
+            runner = CliRunner(mix_stderr=False)
+        except TypeError:
+            # Fallback for older Click versions (or unexpected signature)
+            print("TypeError with mix_stderr=False, falling back to echo_stdin=True")
+            runner = CliRunner(echo_stdin=True)
 
         with runner.isolated_filesystem():
             create_project("pyproject.toml", main_branch="master")
@@ -254,10 +301,9 @@ class TestChecker(TestCase):
             with open(fragment_path, "w") as f:
                 f.write("Adds gravity back")
 
-            call(["git", "add", fragment_path])
-            call(["git", "commit", "-m", "add a newsfragment"])
+            check_call(["git", "add", fragment_path])
+            check_call(["git", "commit", "-m", "add a newsfragment"])
 
-            runner = CliRunner(mix_stderr=False)
             result = runner.invoke(towncrier_check, ["--compare-with", "master"])
 
         self.assertEqual(0, result.exit_code)
@@ -310,16 +356,18 @@ class TestChecker(TestCase):
             commit("First release")
             # The news file is now created.
             self.assertIn("NEWS.rst", os.listdir("."))
-            call(["git", "checkout", "main"])
-            call(["git", "merge", "otherbranch", "-m", "Sync release in main branch."])
+            check_call(["git", "checkout", "main"])
+            check_call(
+                ["git", "merge", "otherbranch", "-m", "Sync release in main branch."]
+            )
 
             # We have a new feature branch that has a news fragment that
             # will be merged to the main branch.
-            call(["git", "checkout", "-b", "new-feature-branch"])
+            check_call(["git", "checkout", "-b", "new-feature-branch"])
             write("foo/newsfragments/456.feature", "Foo the bar")
             commit("A feature in the second release.")
-            call(["git", "checkout", "main"])
-            call(
+            check_call(["git", "checkout", "main"])
+            check_call(
                 [
                     "git",
                     "merge",
@@ -330,7 +378,7 @@ class TestChecker(TestCase):
             )
 
             # We now have the new release branch.
-            call(["git", "checkout", "-b", "next-release"])
+            check_call(["git", "checkout", "-b", "next-release"])
             runner.invoke(towncrier_build, ["--yes", "--version", "2.0"])
             commit("Second release")
 
@@ -355,24 +403,6 @@ class TestChecker(TestCase):
         self.assertEqual(1, result.exit_code)
         self.assertEqual("Could not detect default branch. Aborting.\n", result.output)
 
-    def test_get_default_compare_branch_main(self):
-        """
-        If there's a remote branch origin/main, prefer it over everything else.
-        """
-        branch = check._get_default_compare_branch(["origin/master", "origin/main"])
-
-        self.assertEqual("origin/main", branch)
-
-    def test_get_default_compare_branch_fallback(self):
-        """
-        If there's origin/master and no main, use it and warn about it.
-        """
-        with warnings.catch_warnings(record=True) as w:
-            branch = check._get_default_compare_branch(["origin/master", "origin/foo"])
-
-        self.assertEqual("origin/master", branch)
-        self.assertTrue(w[0].message.args[0].startswith('Using "origin/master'))
-
     @with_isolated_runner
     def test_in_different_dir_with_nondefault_newsfragments_directory(self, runner):
         """
@@ -392,7 +422,7 @@ class TestChecker(TestCase):
         (subproject1 / "changelog.d").mkdir(parents=True)
         (subproject1 / "changelog.d/123.feature").write_text("Adds levitation")
         initial_commit(branch=main_branch)
-        call(["git", "checkout", "-b", "otherbranch"])
+        check_call(["git", "checkout", "-b", "otherbranch"])
 
         # We add a code change but forget to add a news fragment.
         write(subproject1 / "foo/somefile.py", "import os")
@@ -475,8 +505,12 @@ class TestChecker(TestCase):
     def test_ignored_files(self, runner):
         """
         When `ignore` is set in config, files with those names are ignored.
+        Configuration supports wildcard matching with `fnmatch`.
         """
-        create_project("pyproject.toml", extra_config='ignore = ["template.jinja"]')
+        create_project(
+            "pyproject.toml",
+            extra_config='ignore = ["template.jinja", "star_wildcard*"]',
+        )
 
         write(
             "foo/newsfragments/124.feature",
@@ -484,6 +518,7 @@ class TestChecker(TestCase):
         )
         write("foo/newsfragments/template.jinja", "This is manually ignored")
         write("foo/newsfragments/.gitignore", "gitignore is automatically ignored")
+        write("foo/newsfragments/star_wildcard_foo", "Manually ignored with * wildcard")
         commit("add stuff")
 
         result = runner.invoke(towncrier_check, ["--compare-with", "main"])
@@ -514,3 +549,69 @@ class TestChecker(TestCase):
         result = runner.invoke(towncrier_check, ["--compare-with", "main"])
         self.assertEqual(1, result.exit_code, result.output)
         self.assertIn("Invalid news fragment name: feature.125", result.output)
+
+    @with_isolated_runner
+    def test_issue_pattern(self, runner):
+        """
+        Fails if an issue name goes against the configured pattern.
+        """
+        create_project(
+            "pyproject.toml",
+            extra_config='issue_pattern = "\\\\d+"',
+        )
+        write(
+            "foo/newsfragments/123.feature",
+            "This fragment has a valid name",
+        )
+        write(
+            "foo/newsfragments/+abcdefg.feature",
+            "This fragment has a valid name (orphan fragment)",
+        )
+        commit("add stuff")
+
+        result = runner.invoke(towncrier_check, ["--compare-with", "main"])
+        self.assertEqual(0, result.exit_code, result.output)
+
+    @with_isolated_runner
+    def test_issue_pattern_invalid_with_suffix(self, runner):
+        """
+        Fails if an issue name goes against the configured pattern.
+        """
+        create_project(
+            "pyproject.toml",
+            extra_config='issue_pattern = "\\\\d+"',
+        )
+        write(
+            "foo/newsfragments/AAA.BBB.feature.md",
+            "This fragment has an invalid name (should be digits only)",
+        )
+        commit("add stuff")
+
+        result = runner.invoke(towncrier_check, ["--compare-with", "main"])
+        self.assertEqual(1, result.exit_code, result.output)
+        self.assertIn(
+            "Error: Issue name 'AAA.BBB' does not match the configured pattern, '\\d+'",
+            result.output,
+        )
+
+    @with_isolated_runner
+    def test_issue_pattern_invalid(self, runner):
+        """
+        Fails if an issue name goes against the configured pattern.
+        """
+        create_project(
+            "pyproject.toml",
+            extra_config='issue_pattern = "\\\\d+"',
+        )
+        write(
+            "foo/newsfragments/AAA.BBB.feature",
+            "This fragment has an invalid name (should be digits only)",
+        )
+        commit("add stuff")
+
+        result = runner.invoke(towncrier_check, ["--compare-with", "main"])
+        self.assertEqual(1, result.exit_code, result.output)
+        self.assertIn(
+            "Error: Issue name 'AAA.BBB' does not match the configured pattern, '\\d+'",
+            result.output,
+        )
